
#include <stdio.h>

#include <string.h>

#include <dos.h>


typedef unsigned char uint8_t;

typedef unsigned short uint16_t;

typedef unsigned long uint32_t;


#pragma pack(1)


struct FarPointer

{

  uint16_t offset;

  uint16_t segment;

};


struct vbeinfoblock  {

  char VbeSignature[4];

  uint16_t VbeVersion;

  struct FarPointer OemStringPtr;

  uint32_t Capabilities;

  struct FarPointer VideoModePtr;

  uint16_t TotalMemory;



  uint16_t OemSoftwareRev;

  struct FarPointer OemVendorNamePtr;

  struct FarPointer OemProductNamePtr;

  struct FarPointer OemProductRevPtr;

  uint8_t Reserved1[222];


  uint8_t OemData[256];

};


#pragma pack()


void check_size_and_offsets();


int main()

{

  struct vbeinfoblock VIB;

  union REGS inregs;

  union REGS outregs;

  struct SREGS sregs;

  unsigned ax_reg = 0;

  char far* ptr1 = 0; 


  check_size_and_offsets();

  

  memset(&VIB, 0, sizeof(VIB));

  memset(&inregs, 0, sizeof(inregs));

  memset(&outregs, 0, sizeof(outregs));

  memset(&sregs, 0, sizeof(sregs));


  memcpy(VIB.VbeSignature, "VBE2", 4);

  printf("Init signature: %.4s\n", VIB.VbeSignature);


  inregs.x.ax = 0x4F00;

  inregs.x.di = FP_OFF(&VIB);

  sregs.es = FP_SEG(&VIB);

  ax_reg = int86x(0x10, &inregs, &outregs, &sregs);


  if(ax_reg != 0x004F)

  {

    printf("no VESA #1\n");

    return 1;

  }



  if(memcmp(VIB.VbeSignature, "VESA", 4) != 0)

  {

    printf("no VESA #2\n");

    return 1;

  }
  
  printf("  Signature: %.4s\n", VIB.VbeSignature);

  printf("  VbeVersion: 0x%04X\n", VIB.VbeVersion);
  
  ptr1 = MK_FP(VIB.OemStringPtr.segment, VIB.OemStringPtr.offset);

  printf("  OemStringPtr: %s\n", ptr1);

  printf("  Capabilities: 0x%08X\n", VIB.Capabilities);

  printf("  VideoModePtr: 0x%08X\n", VIB.VideoModePtr);

  printf("  TotalMemory: %4d KB\n", VIB.TotalMemory*64);

  printf("---VBE2---\n");

  ptr1 = MK_FP(VIB.OemVendorNamePtr.segment, VIB.OemVendorNamePtr.offset);

  printf("  OemVendorNamePtr: %s\n", ptr1);

  ptr1 = MK_FP(VIB.OemProductNamePtr.segment, VIB.OemProductNamePtr.offset);

  printf("  OemVendorNamePtr: %s\n", ptr1);


  return 0;

}


void check_size_and_offsets()

{

#define STATIC_ASSERT(COND) {char static_assertion[(COND)?1:-1];}

#define OFFSETOF(type, field)((unsigned long) &(((type *) 0)->field))


  STATIC_ASSERT(sizeof(uint8_t)==1);

  STATIC_ASSERT(sizeof(uint16_t)==2);

  STATIC_ASSERT(sizeof(uint32_t)==4);

  

  STATIC_ASSERT(sizeof(struct vbeinfoblock)==512);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, VbeSignature)==0x00);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, VbeVersion)==0x04);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, OemStringPtr)==0x06);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, Capabilities)==0x0A);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, VideoModePtr)==0x0E);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, TotalMemory)==0x12);  

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, OemSoftwareRev)==0x14);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, OemVendorNamePtr)==0x16);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, OemProductNamePtr)==0x1A);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, OemProductRevPtr)==0x1E);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, Reserved1)==0x22);

  STATIC_ASSERT(OFFSETOF(struct vbeinfoblock, OemData)==0x100);

}